import { ɵɵdefineInjectable, Injectable, EventEmitter, Component, Input, Output, ɵɵinject, ViewChild, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { CommonModule } from '@angular/common';
import { __awaiter } from 'tslib';
import { load } from '@tensorflow-models/blazeface';
import { load as load$1 } from '@tensorflow-models/mobilenet';
import { getBackend } from '@tensorflow/tfjs';

var Error;
(function (Error) {
    Error[Error["CONFIGURATION"] = -1] = "CONFIGURATION";
    Error[Error["CAMERA"] = -2] = "CAMERA";
    Error[Error["PREDICTION"] = -3] = "PREDICTION";
    Error[Error["BLAZE_FACE"] = -4] = "BLAZE_FACE";
    Error[Error["MOBILE_NET"] = -5] = "MOBILE_NET";
    Error[Error["IMAGE"] = -6] = "IMAGE";
    Error[Error["CHARGE"] = -7] = "CHARGE";
})(Error || (Error = {}));

const PsmErrorDescription = {
    CONFIGURATION: { code: Error.CONFIGURATION, error: 'La imágen de la compañía es requerida' },
    CAMERA: { code: Error.CAMERA, error: 'Error en la cámara' },
    TENSORFLOW_PREDICTION: { code: Error.PREDICTION, error: 'Error en la predicción de tensorflow' },
    BLAZE_FACE: { code: Error.BLAZE_FACE, error: 'Error al inicializar modelo blaceface' },
    MOBILE_NET: { code: Error.MOBILE_NET, error: 'Error al inicializar modelo MOBILNET' },
    IMAGE_ERROR: { code: Error.IMAGE, error: 'Error al cargar la imagen' },
    CHARGE: { code: Error.CHARGE, error: 'No se han cargado todos los elementos de tensorflow' },
};

class EventService {
    constructor() {
        // loader
        this.counter = 0;
        this.loaderState = new Subject();
        // error
        this.errorState = new Subject();
        // data
        this.dataState = new Subject();
    }
    // loader
    displayLoader(show) {
        show ? this.counter++ : this.counter--;
        this.loaderState.next(this.counter ? true : false);
    }
    get verifyStateLoader() {
        return this.loaderState.asObservable();
    }
    // error
    displayError(error) {
        this.errorState.next(error);
    }
    get verifyStateError() {
        return this.errorState.asObservable();
    }
    // data
    displayData(data) {
        this.dataState.next(data);
    }
    get verifyStateData() {
        return this.dataState.asObservable();
    }
}
EventService.ɵprov = ɵɵdefineInjectable({ factory: function EventService_Factory() { return new EventService(); }, token: EventService, providedIn: "root" });
EventService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
EventService.ctorParameters = () => [];

class PsmFaceCaptchaComponent {
    constructor(eventService) {
        this.eventService = eventService;
        this.backgroundIcon = 'white';
        this.switchCamera = true;
        this.onerror = new EventEmitter();
        this.onloader = new EventEmitter();
        this.ondata = new EventEmitter();
        this.showCamera = true;
    }
    ngOnInit() {
        this.getLoaderState();
        this.getErrorState();
        this.getDataState();
        this.validateInputData();
    }
    validateInputData() {
        if (!this.companySrc) {
            this.showCamera = false;
            this.eventService.displayError(PsmErrorDescription.CONFIGURATION);
            return;
        }
    }
    getLoaderState() {
        this.loaderService = this.eventService.verifyStateLoader.subscribe(res => {
            this.onloader.emit(res);
        });
    }
    getErrorState() {
        this.errorService = this.eventService.verifyStateError.subscribe(res => {
            this.onerror.emit(res);
        });
    }
    getDataState() {
        this.dataService = this.eventService.verifyStateData.subscribe(res => {
            this.showCamera = false;
            this.ondata.emit(res);
        });
    }
    ngOnDestroy() {
        if (this.loaderService)
            this.loaderService.unsubscribe();
        if (this.errorService)
            this.errorService.unsubscribe();
        if (this.dataService)
            this.dataService.unsubscribe();
    }
}
PsmFaceCaptchaComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-psm-face-captcha',
                template: "<lib-camera *ngIf=\"showCamera\" [backgroundIcon]=\"backgroundIcon\" [companySrc]=\"companySrc\"></lib-camera>\r\n",
                styles: [":host{box-sizing:border-box;display:block;height:100%;left:0;position:fixed;top:0;width:100%}"]
            },] }
];
PsmFaceCaptchaComponent.ctorParameters = () => [
    { type: EventService }
];
PsmFaceCaptchaComponent.propDecorators = {
    companySrc: [{ type: Input }],
    backgroundIcon: [{ type: Input }],
    switchCamera: [{ type: Input }],
    onerror: [{ type: Output }],
    onloader: [{ type: Output }],
    ondata: [{ type: Output }]
};

var Image;
(function (Image) {
    Image[Image["WIDTH"] = 640] = "WIDTH";
    Image[Image["HEIGHT"] = 480] = "HEIGHT";
})(Image || (Image = {}));

class TensorflowService {
    constructor(eventService) {
        this.eventService = eventService;
        getBackend();
        this.loadBlazeFace();
        this.loadMobilenet();
    }
    loadBlazeFace() {
        return __awaiter(this, void 0, void 0, function* () {
            this.eventService.displayLoader(true);
            this.blazefaceModel = yield load().then(res => res).catch(() => null);
            if (!this.blazefaceModel)
                this.eventService.displayError(PsmErrorDescription.TENSORFLOW_PREDICTION);
            this.eventService.displayLoader(false);
        });
    }
    loadMobilenet() {
        return __awaiter(this, void 0, void 0, function* () {
            this.eventService.displayLoader(true);
            this.mobilenetModel = yield load$1().then(res => res).catch(() => null);
            if (!this.mobilenetModel)
                this.eventService.displayError(PsmErrorDescription.MOBILE_NET);
            this.eventService.displayLoader(false);
        });
    }
    getPrediction(element) {
        return __awaiter(this, void 0, void 0, function* () {
            this.eventService.displayLoader(true);
            try {
                const face = yield this.blazefaceModel.estimateFaces(element, false);
                const data = yield this.mobilenetModel.classify(element);
                this.eventService.displayLoader(false);
                return {
                    hasFace: face && face.length > 0 ? true : false,
                    facesDetected: face.length,
                    objectPrediction: data,
                    image: null
                };
            }
            catch (error) {
                this.eventService.displayLoader(false);
                return null;
            }
        });
    }
    getPredictionData(src) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.blazefaceModel || !this.mobilenetModel) {
                this.eventService.displayError(PsmErrorDescription.CHARGE);
                return;
            }
            const img = yield this.getImage(src).then(res => res);
            if (img) {
                const data = yield this.getPrediction(img);
                if (data)
                    this.eventService.displayData(data);
                else
                    this.eventService.displayError(PsmErrorDescription.TENSORFLOW_PREDICTION);
            }
            else
                this.eventService.displayError(PsmErrorDescription.IMAGE_ERROR);
        });
    }
    getImage(src) {
        return new Promise((resolve, reject) => {
            const img = document.createElement('img');
            img.id = 'psm-img';
            img.src = src;
            img.onload = () => {
                console.log('leído');
                resolve(img);
            };
            img.onerror = () => {
                resolve(null);
            };
        });
    }
}
TensorflowService.ɵprov = ɵɵdefineInjectable({ factory: function TensorflowService_Factory() { return new TensorflowService(ɵɵinject(EventService)); }, token: TensorflowService, providedIn: "root" });
TensorflowService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
TensorflowService.ctorParameters = () => [
    { type: EventService }
];

class CameraComponent {
    constructor(eventService, tensorflowService) {
        this.eventService = eventService;
        this.tensorflowService = tensorflowService;
        this.constraints = {
            video: {
                width: Image.WIDTH,
                height: Image.HEIGHT,
                facingMode: 'user'
            },
            audio: false
        };
    }
    ngAfterViewInit() {
        return __awaiter(this, void 0, void 0, function* () {
            this.eventService.displayLoader(true);
            yield this.getUserMedia(this.constraints);
            this.eventService.displayLoader(false);
        });
    }
    takepicture() {
        return __awaiter(this, void 0, void 0, function* () {
            this.canvas.nativeElement.getContext("2d").drawImage(this.video.nativeElement, 0, 0, 640, 480);
            const photo = this.canvas.nativeElement.toDataURL("image/png");
            this.video.nativeElement.pause();
            this.video.nativeElement.srcObject = null;
            this.stream.getTracks().forEach(track => track.stop());
            const prediction = yield this.tensorflowService.getPrediction(this.canvas.nativeElement);
            if (prediction) {
                prediction.image = photo;
                this.eventService.displayData(prediction);
            }
            else
                this.eventService.displayError(PsmErrorDescription.TENSORFLOW_PREDICTION);
        });
    }
    switchCamera() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.constraints.video.facingMode === 'user')
                this.constraints.video.facingMode = 'environment';
            else
                this.constraints.video.facingMode = 'user';
            yield this.getUserMedia(this.constraints);
        });
    }
    getUserMedia(constraints) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.stream) {
                this.video.nativeElement.pause();
                this.video.nativeElement.srcObject = null;
                this.stream.getTracks().forEach(track => track.stop());
            }
            const stream = yield navigator.mediaDevices.getUserMedia(constraints).then((stream) => stream).catch(() => null);
            if (stream) {
                this.stream = stream;
                this.video.nativeElement.srcObject = stream;
                this.video.nativeElement.play();
            }
            else
                this.eventService.displayError(PsmErrorDescription.CAMERA);
        });
    }
    ngOnDestroy() {
        if (this.stream)
            this.stream.getTracks().forEach(track => track.stop());
    }
}
CameraComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-camera',
                template: "\n<video #video playsinline [muted]=\"'muted'\" [width]=\"constraints.video.width\" [height]=\"constraints.video.height\"></video>\n<div class=\"container-company\"><img [src]=\"companySrc\" alt=\"company\"></div>\n<lib-capture [backgroundIcon]=\"backgroundIcon\" (click)=\"takepicture()\"></lib-capture>\n<lib-switch [backgroundIcon]=\"backgroundIcon\" (click)=\"switchCamera()\"></lib-switch>\n<canvas #canvas [width]=\"constraints.video.width\" [height]=\"constraints.video.height\"></canvas>\n",
                styles: [":host,:host video{display:block;height:100%;width:100%}:host video{object-fit:cover}:host canvas{display:none}:host .container-company{height:100px;position:fixed;right:3%;top:3%;width:100px}:host .container-company img{display:block;margin-left:auto;max-height:100%;max-width:100%}"]
            },] }
];
CameraComponent.ctorParameters = () => [
    { type: EventService },
    { type: TensorflowService }
];
CameraComponent.propDecorators = {
    backgroundIcon: [{ type: Input }],
    companySrc: [{ type: Input }],
    video: [{ type: ViewChild, args: ["video",] }],
    canvas: [{ type: ViewChild, args: ["canvas",] }]
};

class CaptureComponent {
}
CaptureComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-capture',
                template: "<svg aria-hidden=\"true\" focusable=\"false\" data-prefix=\"fas\" data-icon=\"camera\" class=\"svg-inline--fa fa-camera fa-w-16\"\n  role=\"img\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 512 512\">\n  <path [attr.fill]=\"backgroundIcon\"\n    d=\"M512 144v288c0 26.5-21.5 48-48 48H48c-26.5 0-48-21.5-48-48V144c0-26.5 21.5-48 48-48h88l12.3-32.9c7-18.7 24.9-31.1 44.9-31.1h125.5c20 0 37.9 12.4 44.9 31.1L376 96h88c26.5 0 48 21.5 48 48zM376 288c0-66.2-53.8-120-120-120s-120 53.8-120 120 53.8 120 120 120 120-53.8 120-120zm-32 0c0 48.5-39.5 88-88 88s-88-39.5-88-88 39.5-88 88-88 88 39.5 88 88z\">\n  </path>\n</svg>\n",
                styles: [":host{bottom:10px;left:50%;position:fixed;transform:translate(-50%);width:50px;z-index:2}"]
            },] }
];
CaptureComponent.propDecorators = {
    backgroundIcon: [{ type: Input }]
};

class SwitchComponent {
}
SwitchComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-switch',
                template: "<svg aria-hidden=\"true\" focusable=\"false\" data-prefix=\"fas\" data-icon=\"retweet\"\n  class=\"svg-inline--fa fa-retweet fa-w-20\" role=\"img\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 640 512\">\n  <path [attr.fill]=\"backgroundIcon\"\n    d=\"M629.657 343.598L528.971 444.284c-9.373 9.372-24.568 9.372-33.941 0L394.343 343.598c-9.373-9.373-9.373-24.569 0-33.941l10.823-10.823c9.562-9.562 25.133-9.34 34.419.492L480 342.118V160H292.451a24.005 24.005 0 0 1-16.971-7.029l-16-16C244.361 121.851 255.069 96 276.451 96H520c13.255 0 24 10.745 24 24v222.118l40.416-42.792c9.285-9.831 24.856-10.054 34.419-.492l10.823 10.823c9.372 9.372 9.372 24.569-.001 33.941zm-265.138 15.431A23.999 23.999 0 0 0 347.548 352H160V169.881l40.416 42.792c9.286 9.831 24.856 10.054 34.419.491l10.822-10.822c9.373-9.373 9.373-24.569 0-33.941L144.971 67.716c-9.373-9.373-24.569-9.373-33.941 0L10.343 168.402c-9.373 9.373-9.373 24.569 0 33.941l10.822 10.822c9.562 9.562 25.133 9.34 34.419-.491L96 169.881V392c0 13.255 10.745 24 24 24h243.549c21.382 0 32.09-25.851 16.971-40.971l-16.001-16z\">\n  </path>\n</svg>\n",
                styles: [":host{bottom:10px;position:fixed;right:5%;width:60px;z-index:2}"]
            },] }
];
SwitchComponent.propDecorators = {
    backgroundIcon: [{ type: Input }]
};

class PsmFaceCaptchaModule {
}
PsmFaceCaptchaModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    PsmFaceCaptchaComponent,
                    CameraComponent,
                    CaptureComponent,
                    SwitchComponent
                ],
                imports: [
                    CommonModule
                ],
                exports: [PsmFaceCaptchaComponent]
            },] }
];

class PsmCaptchaService {
    constructor(eventService, tensorflowService) {
        this.eventService = eventService;
        this.tensorflowService = tensorflowService;
    }
    getEvents() {
        return {
            loader: this.eventService.verifyStateLoader,
            error: this.eventService.verifyStateError,
            data: this.eventService.verifyStateData,
        };
    }
    getPrediction(src) {
        this.tensorflowService.getPredictionData(src);
    }
}
PsmCaptchaService.ɵprov = ɵɵdefineInjectable({ factory: function PsmCaptchaService_Factory() { return new PsmCaptchaService(ɵɵinject(EventService), ɵɵinject(TensorflowService)); }, token: PsmCaptchaService, providedIn: "root" });
PsmCaptchaService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
PsmCaptchaService.ctorParameters = () => [
    { type: EventService },
    { type: TensorflowService }
];

/*
 * Public API Surface of psm-face-captcha
 */

/**
 * Generated bundle index. Do not edit.
 */

export { PsmCaptchaService, PsmFaceCaptchaComponent, PsmFaceCaptchaModule, EventService as ɵa, CameraComponent as ɵb, TensorflowService as ɵc, CaptureComponent as ɵd, SwitchComponent as ɵe };
//# sourceMappingURL=psm-face-captcha.js.map
